@@ -109,12 +109,15 @@ def admin_get_queryset(admin, request, fields=(), debug=False):
     if debug:
         return DebugQS(f"{admin}.get_queryset()")
     else:
-        request.data_browser = {"calculated_fields": set(fields), "fields": set(fields)}
+        request.data_browser.update(
+            {"calculated_fields": set(fields), "fields": set(fields)}
+        )
         return admin.get_queryset(request)
 
 
 def admin_get_actions(admin, request):
-    request.data_browser = {"calculated_fields": set(), "fields": set()}
+    assert hasattr(request, "data_browser"), request
+
     res = {}
     for func, name, desc in admin.get_actions(request).values():
         if not getattr(func, "ddb_hide", False):
@@ -132,7 +135,7 @@ def _model_has_field(model, field_name):
 
 
 def _get_all_admin_fields(request):
-    request.data_browser = {"calculated_fields": set(), "fields": set()}
+    assert hasattr(request, "data_browser"), request
 
     def from_fieldsets(admin, include_calculated):
         auth_user_compat = settings.DATA_BROWSER_AUTH_USER_COMPAT
@@ -153,7 +156,8 @@ def _get_all_admin_fields(request):
         )
         if not has_attrs or not issubclass(model_admin.model, models.Model):
             debug_log(
-                f"{type(model_admin)} instance does not look like a ModelAdmin or InlineModelAdmin"
+                f"{type(model_admin)} instance does not look like a ModelAdmin or"
+                " InlineModelAdmin"
             )
             return False
 
@@ -259,7 +263,8 @@ def _get_calculated_field(request, field_name, model_name, model, admin, model_f
         field = getattr(annotation, "output_field", None)
         if not field:  # pragma: no cover
             raise Exception(
-                f"Annotation '{field_name}' for {admin}.{field_name} doesn't specify 'output_field'"
+                f"Annotation '{field_name}' for {admin}.{field_name} doesn't specify"
+                " 'output_field'"
             )
 
         type_, choices = get_field_type(field_name, field)
@@ -323,7 +328,7 @@ def _get_fields_for_model(request, model, admin, admin_fields):
         pretty_name = (
             _make_pretty(getattr(field, "verbose_name", field_name)) if field else None
         )
-        # FK's and OneToOne's
+        # FK's etc
         if isinstance(
             field, (models.ForeignKey, ForeignObjectRel, models.ManyToManyField)
         ):
@@ -332,8 +337,7 @@ def _get_fields_for_model(request, model, admin, admin_fields):
                 continue
 
             if isinstance(field, ForeignObjectRel):
-                field_name = field.get_accessor_name()
-                pretty_name = _make_pretty(field_name)
+                pretty_name = _make_pretty(field.get_accessor_name())
 
             if field.related_model in admin_fields:
                 fields[field_name] = OrmFkField(
