@@ -33,18 +33,25 @@ class TypeMeta(type):
     def lookups(cls):
         res = {}
         for name, type_ in cls._lookups().items():
-            if isinstance(type_, tuple):
-                pretty_name, type_ = type_
+            assert isinstance(type_, tuple), type_
+            if len(type_) == 3:
+                type_, keep_choices, pretty_name = type_
+                assert keep_choices is not None, (cls, name)
             else:
+                type_, keep_choices = type_
                 pretty_name = name.replace("_", " ")
-            res[name] = pretty_name, type_.name
+            res[name] = pretty_name, type_.name, keep_choices
         return res
 
+    @property
+    def pretty_name(cls):
+        name = cls.__name__
+        assert name.endswith("Type")
+        return name[: -len("Type")]
+
     @property
     def name(cls):
-        name = cls.__name__.lower()
-        assert name.endswith("type")
-        return name[: -len("type")]
+        return cls.pretty_name.lower()
 
 
 class BaseType(metaclass=TypeMeta):
@@ -58,7 +65,11 @@ class BaseType(metaclass=TypeMeta):
 
     @classmethod
     def _lookups(cls):
-        return {"equals": cls, "not_equals": cls, "is_null": IsNullType}
+        return {
+            "equals": (cls, True),
+            "not_equals": (cls, True),
+            "is_null": (IsNullType, False),
+        }
 
     @staticmethod
     def _get_formatter(choices):
@@ -71,6 +82,7 @@ class BaseType(metaclass=TypeMeta):
 
     @staticmethod
     def _parse(value, choices):
+        assert not choices
         return value
 
     @classmethod
@@ -88,13 +100,17 @@ class BaseType(metaclass=TypeMeta):
         if lookup not in lookups:
             return None, "Bad lookup"
         else:
-            pretty_name, type_name = lookups[lookup]
+            pretty_name, type_name, keep_choices = lookups[lookup]
+            if not keep_choices:
+                choices = ()
             type_ = TYPES[type_name]
             return type_.parse(value, choices)
 
     @classmethod
     def format_lookup(cls, lookup, value, choices):
-        pretty_name, type_name = cls.lookups[lookup]
+        pretty_name, type_name, keep_choices = cls.lookups[lookup]
+        if not keep_choices:
+            choices = ()
         return TYPES[type_name]._get_formatter(choices)(value)
 
     @staticmethod
@@ -108,35 +124,46 @@ class StringType(BaseType):
     @classmethod
     def _lookups(cls):
         return {
-            "equals": cls,
-            "contains": cls,
-            "starts_with": cls,
-            "ends_with": cls,
-            "regex": RegexType,
-            "not_equals": cls,
-            "not_contains": cls,
-            "not_starts_with": cls,
-            "not_ends_with": cls,
-            "not_regex": RegexType,
-            "is_null": IsNullType,
+            "equals": (cls, True),
+            "contains": (cls, True),
+            "starts_with": (cls, True),
+            "ends_with": (cls, True),
+            "regex": (RegexType, False),
+            "not_equals": (cls, True),
+            "not_contains": (cls, True),
+            "not_starts_with": (cls, True),
+            "not_ends_with": (cls, True),
+            "not_regex": (RegexType, False),
+            "is_null": (IsNullType, False),
         }
 
 
-class NumberType(BaseType):
-    default_value = 0
+class StringableType(BaseType):
+    default_value = ""
 
+    @staticmethod
+    def _get_formatter(choices):  # pragma: no cover
+        assert not choices
+        return lambda value: None if value is None else str(value)
+
+
+class SequenceTypeMixin:
     @classmethod
     def _lookups(cls):
         return {
-            "equals": cls,
-            "not_equals": cls,
-            "gt": (">", cls),
-            "gte": (">=", cls),
-            "lt": ("<", cls),
-            "lte": ("<=", cls),
-            "is_null": IsNullType,
+            "equals": (cls, True),
+            "not_equals": (cls, True),
+            "gt": (cls, True, ">"),
+            "gte": (cls, True, ">="),
+            "lt": (cls, True, "<"),
+            "lte": (cls, True, "<="),
+            "is_null": (IsNullType, False),
         }
 
+
+class NumberType(SequenceTypeMixin, BaseType):
+    default_value = 0
+
     @staticmethod
     def _get_formatter(choices):
         assert not choices
@@ -144,6 +171,7 @@ class NumberType(BaseType):
 
     @staticmethod
     def _parse(value, choices):
+        assert not choices
         return float(value)
 
     @staticmethod
@@ -167,6 +195,7 @@ class RegexType(BaseType):
     @staticmethod
     @lru_cache(maxsize=None)
     def _parse(value, choices):
+        assert not choices
         from django.contrib.contenttypes.models import ContentType
         from django.db.transaction import atomic
 
@@ -182,23 +211,13 @@ class RegexType(BaseType):
         return value
 
 
-class DurationType(BaseType):
+class DurationType(SequenceTypeMixin, BaseType):
     default_value = ""
 
-    @classmethod
-    def _lookups(cls):
-        return {
-            "equals": cls,
-            "not_equals": cls,
-            "gt": (">", cls),
-            "gte": (">=", cls),
-            "lt": ("<", cls),
-            "lte": ("<=", cls),
-            "is_null": IsNullType,
-        }
-
     @staticmethod
     def _parse(value, choices):
+        assert not choices
+
         if value.count(":") == 1:
             value += ":0"
 
@@ -213,7 +232,7 @@ class DurationType(BaseType):
         return lambda value: None if value is None else str(value)
 
 
-class DateTypeMixin:
+class DateTimeParseMixin:
     _clause = re.compile(r"(\w{3,})([-+=])(\d+) *")
 
     _clause_types = {
@@ -238,109 +257,105 @@ class DateTypeMixin:
         for c2 in _clause_types:
             assert c1 == c2 or not c1.startswith(c2), c2
 
-    @classmethod
-    def _lookups(cls):
-        return {
-            "equals": cls,
-            "not_equals": cls,
-            "gt": (">", cls),
-            "gte": (">=", cls),
-            "lt": ("<", cls),
-            "lte": ("<=", cls),
-            "is_null": IsNullType,
-        }
+    @staticmethod
+    def _unambiguous_date_parse(value, **kwargs):
+        try:
+            res = {
+                dateutil.parser.parse(value, dayfirst=False, yearfirst=False, **kwargs),
+                dateutil.parser.parse(value, dayfirst=True, yearfirst=False, **kwargs),
+                dateutil.parser.parse(value, dayfirst=False, yearfirst=True, **kwargs),
+                dateutil.parser.parse(value, dayfirst=True, yearfirst=True, **kwargs),
+            }
+            if len(res) != 1:
+                raise Exception("Ambiguous value")
+            return res.pop()
+        except dateutil.parser.ParserError:
+            return None
+
+    @staticmethod
+    def _truncate_dt(value):
+        return value.replace(hour=0, minute=0, second=0, microsecond=0)
 
     @classmethod
     def _parse(cls, value, choices):
-        value = value.strip()
+        assert not choices
 
         d8 = r"(\d{8})"
         d422 = r"(\d{4}[^\d]+\d{1,2}[^\d]+\d{1,2}([^\d]|$))"
-        if re.match(r"[^\d]*(" + d8 + "|" + d422 + ")", value):
-            # looks like some kinda iso date, roll with the defaults
-            res = dateutil.parser.parse(value)
-        else:
-            try:
-                # not an iso date, let the parser do it thing, but check for ambiguities
-                res = {
-                    dateutil.parser.parse(value, dayfirst=False, yearfirst=False),
-                    dateutil.parser.parse(value, dayfirst=True, yearfirst=False),
-                    dateutil.parser.parse(value, dayfirst=False, yearfirst=True),
-                    dateutil.parser.parse(value, dayfirst=True, yearfirst=True),
-                }
-                if len(res) != 1:
-                    raise Exception("Ambiguous value")
-                res = res.pop()
-            except dateutil.parser.ParserError:
-                # failing that must be relative delta stuff
+
+        res = timezone.now()
+
+        # iterate the clauses in the expression
+        for clause_str in value.split():
+            clause_str = clause_str.lower()
+
+            if clause_str == "now":
                 res = timezone.now()
+            elif clause_str == "today":
+                res = cls._truncate_dt(timezone.now())
+            elif re.match(fr"[^\d]*({d8}|{d422})", clause_str):
+                # looks like some kinda iso date, roll with the defaults
+                # includes T delimited like 2018-03-20T22:31:23
+                res = dateutil.parser.parse(clause_str, default=cls._truncate_dt(res))
+            elif cls._unambiguous_date_parse(clause_str, default=cls._truncate_dt(res)):
+                # TODO could walrus the duplicate call once we drop support for py3.7
+                res = cls._unambiguous_date_parse(
+                    clause_str, default=cls._truncate_dt(res)
+                )
+            else:
+                # failing that must be relative delta stuff
+                match = cls._clause.fullmatch(clause_str)
+                if not match:
+                    raise Exception(f"Unrecognized clause '{clause_str}'")
 
-                # iterate the clauses in the expression
-                for clause_str in value.split():
-                    match = cls._clause.fullmatch(clause_str)
-                    if not match:
-                        raise Exception(f"Unrecognized clause '{clause_str}'")
+                # cut up the clause
+                field, op, val = match.groups()
+                val = int(val)
+                human_op = {"+": "add", "-": "subtract", "=": "set"}[op]
 
-                    # cut up the clause
-                    field, op, val = match.groups()
-                    val = int(val)
-                    human_op = {
-                        "+": "add",
-                        "-": "subtract",
-                        "=": "set",
-                    }[op]
+                # find the field
+                for prefix, arg in cls._clause_types.items():
+                    if field.startswith(prefix):
+                        break
+                else:
+                    raise Exception(f"Unrecognized field '{field}'")
 
-                    # find the field
-                    for prefix, arg in cls._clause_types.items():
-                        if field.startswith(prefix):
-                            break
-                    else:
-                        raise Exception(f"Unrecognized field '{field}'")
-
-                    # year month day etc
-                    if isinstance(arg, str):
-                        if op == "+":
-                            kwargs = {f"{arg}s": val}
-                        elif op == "-":
-                            kwargs = {f"{arg}s": -val}
-                        else:  # op == "="
-                            if arg in ["week"]:
-                                raise Exception(f"'{op}' not supported for '{field}'")
-                            elif arg in ["year", "month", "day"]:
-                                if val <= 0:
-                                    raise Exception(
-                                        f"Can't {human_op} '{field}' to '{val}'"
-                                    )
-                            kwargs = {arg: val}
-
-                    # weekdays
-                    else:
-                        if op == "=":
+                # year month day etc
+                if isinstance(arg, str):
+                    if op == "+":
+                        kwargs = {f"{arg}s": val}
+                    elif op == "-":
+                        kwargs = {f"{arg}s": -val}
+                    else:  # op == "="
+                        if arg in ["week"]:
                             raise Exception(f"'{op}' not supported for '{field}'")
-                        else:
+                        elif arg in ["year", "month", "day"]:
                             if val <= 0:
-                                raise Exception(f"Can't {human_op} '{val}' '{field}'s")
-                            if op == "+":
-                                kwargs = {"weekday": arg(val)}
-                            else:  # op == "-"
-                                kwargs = {"weekday": arg(-val)}
+                                raise Exception(
+                                    f"Can't {human_op} '{field}' to '{val}'"
+                                )
+                        kwargs = {arg: val}
 
-                    res += relativedelta.relativedelta(**kwargs)
-                return res
+                # weekdays
+                else:
+                    if op == "=":
+                        raise Exception(f"'{op}' not supported for '{field}'")
+                    else:
+                        if val <= 0:
+                            raise Exception(f"Can't {human_op} '{val}' '{field}'s")
+                        if op == "+":
+                            kwargs = {"weekday": arg(val)}
+                        else:  # op == "-"
+                            kwargs = {"weekday": arg(-val)}
 
-        return timezone.make_aware(res)
+                res += relativedelta.relativedelta(**kwargs)
+
+        return res
 
 
-class DateTimeType(DateTypeMixin, BaseType):
+class DateTimeType(DateTimeParseMixin, SequenceTypeMixin, BaseType):
     default_value = "now"
 
-    @classmethod
-    def _parse(cls, value, choices):
-        if value.lower().strip() == "now":
-            return timezone.now()
-        else:
-            return super()._parse(value, choices)
-
     @staticmethod
     def _get_formatter(choices):
         assert not choices
@@ -358,16 +373,12 @@ class DateTimeType(DateTypeMixin, BaseType):
             )
 
 
-class DateType(DateTypeMixin, BaseType):
+class DateType(DateTimeParseMixin, SequenceTypeMixin, BaseType):
     default_value = "today"
 
     @classmethod
     def _parse(cls, value, choices):
-        if value.lower().strip() == "today":
-            res = timezone.now()
-        else:
-            res = super()._parse(value, choices)
-        return res.date()
+        return super()._parse(value, choices).date()
 
     @staticmethod
     def _get_formatter(choices):
@@ -378,7 +389,7 @@ class DateType(DateTypeMixin, BaseType):
 class HTMLType(StringType):
     @staticmethod
     def _lookups():
-        return {"is_null": IsNullType}
+        return {"is_null": (IsNullType, False)}
 
     @staticmethod
     def _get_formatter(choices):
@@ -395,6 +406,8 @@ class BooleanType(BaseType):
 
     @staticmethod
     def _parse(value, choices):
+        assert not choices
+
         value = value.lower()
         if value == "true":
             return True
@@ -417,6 +430,8 @@ class UUIDType(BaseType):
 
     @staticmethod
     def _parse(value, choices):
+        assert not choices
+
         return uuid.UUID(value)
 
 
@@ -428,7 +443,7 @@ class UnknownType(BaseType):
 
     @staticmethod
     def _lookups():
-        return {"is_null": IsNullType}
+        return {"is_null": (IsNullType, False)}
 
 
 def _json_loads(value):
@@ -443,6 +458,8 @@ class JSONFieldType(BaseType):
 
     @staticmethod
     def _parse(value, choices):
+        assert not choices
+
         value = value.strip()
         if "|" not in value:
             raise ValueError("Missing seperator '|'")
@@ -456,17 +473,19 @@ class JSONType(BaseType):
     @classmethod
     def _lookups(cls):
         return {
-            "equals": cls,
-            "has_key": StringType,
-            "field_equals": JSONFieldType,
-            "not_equals": cls,
-            "not_has_key": StringType,
-            "not_field_equals": JSONFieldType,
-            "is_null": IsNullType,
+            "equals": (cls, True),
+            "has_key": (StringType, None),
+            "field_equals": (JSONFieldType, None),
+            "not_equals": (cls, True),
+            "not_has_key": (StringType, None),
+            "not_field_equals": (JSONFieldType, None),
+            "is_null": (IsNullType, False),
         }
 
     @staticmethod
     def _parse(value, choices):
+        assert not choices
+
         return _json_loads(value)
 
 
@@ -502,14 +521,16 @@ class IsNullType(ChoiceTypeMixin, BaseType):
 
     @staticmethod
     def _lookups():
-        return {"equals": IsNullType}
+        return {"equals": (IsNullType, False)}
 
     @classmethod
     def _get_formatter(cls, choices):
+        assert not choices
         return super()._get_formatter(cls.choices)
 
     @classmethod
     def _parse(cls, value, choices):
+        assert not choices
         return super()._parse(value, cls.choices)
 
 
@@ -517,7 +538,7 @@ class ArrayTypeMixin:
     default_value = "[]"
 
     @classmethod
-    def _get_formatter(cls, choices):  # pragma: postgres
+    def _get_formatter(cls, choices):
         element_formatter = cls.element_type._get_formatter(choices)
         return (
             lambda value: None
@@ -530,13 +551,13 @@ class ArrayTypeMixin:
     @classmethod
     def _lookups(cls):
         return {
-            "equals": cls,
-            "contains": cls.element_type,
-            "length": NumberType,
-            "not_equals": cls,
-            "not_contains": cls.element_type,
-            "not_length": NumberType,
-            "is_null": IsNullType,
+            "equals": (cls, True),
+            "contains": (cls.element_type, True),
+            "length": (NumberType, False),
+            "not_equals": (cls, True),
+            "not_contains": (cls.element_type, True),
+            "not_length": (NumberType, False),
+            "is_null": (IsNullType, False),
         }
 
     @classmethod
@@ -547,22 +568,31 @@ class ArrayTypeMixin:
         return [cls.element_type._parse(v, choices) for v in value]
 
 
-class StringArrayType(ArrayTypeMixin, BaseType):
-    element_type = StringType
-
-
-class NumberArrayType(ArrayTypeMixin, BaseType):
-    element_type = NumberType
+# Array types
+for _cls in [
+    BooleanType,
+    DurationType,
+    BooleanType,
+    DateTimeType,
+    DateType,
+    UUIDType,
+    URLType,
+    StringType,
+    NumberType,
+    StringableType,
+]:
+    _name = f"{_cls.pretty_name}ArrayType"
+    globals()[_name] = type(_name, (ArrayTypeMixin, BaseType), {"element_type": _cls})
 
 
 class StringChoiceArrayType(ArrayTypeMixin, BaseType):
     element_type = StringChoiceType
-    raw_type = StringArrayType
+    raw_type = StringArrayType  # noqa: F821
 
 
 class NumberChoiceArrayType(ArrayTypeMixin, BaseType):
     element_type = NumberChoiceType
-    raw_type = NumberArrayType
+    raw_type = NumberArrayType  # noqa: F821
 
 
 TYPES = {cls.name: cls for cls in all_subclasses(BaseType)}
