@@ -110,7 +110,8 @@ def _get_config(request):
     types = {
         name: {
             "lookups": {
-                n: {"prettyName": tn, "type": t} for n, (tn, t) in type_.lookups.items()
+                n: {"prettyName": tn, "type": t}
+                for n, (tn, t, _) in type_.lookups.items()
             },
             "sortedLookups": list(type_.lookups),
             "defaultLookup": type_.default_lookup,
@@ -137,8 +138,17 @@ def _get_config(request):
     }
 
 
+def _add_request_info(request, *, view=False):
+    request.data_browser = {
+        "public_view": view,
+        "fields": set(),
+        "calculated_fields": set(),
+    }
+
+
 @admin_decorators.staff_member_required
 def query_ctx(request, *, model_name="", fields=""):
+    _add_request_info(request)
     config = _get_config(request)
     return JsonResponse(config)
 
@@ -177,6 +187,8 @@ def admin_action(request, model_name, fields):
 @csrf.ensure_csrf_cookie
 @admin_decorators.staff_member_required
 def query_html(request, *, model_name="", fields=""):
+    _add_request_info(request)
+
     if request.method == "POST":
         return admin_action(request, model_name, fields)
 
@@ -201,6 +213,8 @@ def query_html(request, *, model_name="", fields=""):
 
 @admin_decorators.staff_member_required
 def query(request, *, model_name, fields="", media):
+    _add_request_info(request)
+
     params = hyperlink.parse(request.get_full_path()).query
 
     if media.startswith("profile") or media.startswith("pstats"):
@@ -232,9 +246,10 @@ def query(request, *, model_name, fields="", media):
                 marshal.dump(pstats.Stats(profiler).stats, buffer)
                 buffer.seek(0)
                 response = HttpResponse(buffer, content_type="application/octet-stream")
-                response[
-                    "Content-Disposition"
-                ] = f"attachment; filename={query.model_name}-{timezone.now().isoformat()}.pstats"
+                response["Content-Disposition"] = (
+                    "attachment;"
+                    f" filename={query.model_name}-{timezone.now().isoformat()}.pstats"
+                )
                 return response
             else:
                 raise http.Http404(f"Bad file format {prof_media} requested")
@@ -247,6 +262,8 @@ def query(request, *, model_name, fields="", media):
 
 
 def view(request, pk, media):
+    _add_request_info(request, view=True)
+
     view = get_object_or_404(View.objects.filter(public=True), public_slug=pk)
     if (
         # some of these are checked by the admin but this is a good time to be paranoid
@@ -303,7 +320,10 @@ def _data_response(request, query, media, privileged=False, strict=False):
         else:
             if media == "sql":
                 res = "This is an approximation of the main query.\n"
-                res += "Pages with pivoted or calculated data may do additional queries.\n\n"
+                res += (
+                    "Pages with pivoted or calculated data may do additional"
+                    " queries.\n\n"
+                )
                 res += sqlparse.format(
                     str(query_set.query), reindent=True, keyword_case="upper"
                 )
@@ -311,7 +331,10 @@ def _data_response(request, query, media, privileged=False, strict=False):
                 res = query_set.explain()
             elif media == "qs":
                 res = "This is an approximation of the main queryset.\n"
-                res += "Pages with pivoted or calculated data may do additional queries.\n\n"
+                res += (
+                    "Pages with pivoted or calculated data may do additional"
+                    " queries.\n\n"
+                )
                 res += str(query_set)
             else:
                 assert False
