@@ -138,15 +138,31 @@ def bound_query(query, orm_models):
     return BoundQuery.bind(query, orm_models)
 
 
-def parse_helper(type_, choices, value, expected, error_re):
-    res, err = type_.parse(value, choices)
-    if expected is None:
-        assert res is None
-        assert re.fullmatch(error_re, err)
-    else:
+class ParseHelpers:
+    choices = ()
+
+    def check_parse(self, value, expected, error_re):
+        res, err = self.type_.parse(value, self.choices)
+        if expected is None:
+            assert res is None
+            assert re.fullmatch(error_re, err)
+        else:
+            assert err is None
+            assert res == expected
+
+    def check_parse_lookup(self, lookup, value, expected, error_re):
+        res, err = self.type_.parse_lookup(lookup, value, self.choices)
+        # if expected is None:
+        #    assert res is None
+        #    assert re.fullmatch(error_re, err)
+        # else:
+        assert expected is not None
         assert err is None
         assert res == expected
 
+    def check_format(self, value, expected):
+        assert self.type_.get_formatter(self.choices)(value) == expected
+
 
 class TestQuery:
     def test_from_request(self, query):
@@ -225,10 +241,10 @@ class TestQuery:
             query.get_url("html")
             == "/query/app.model/fa+1,fd-0,fn.html?bob__equals=fred&limit=123"
         )
-        assert (
-            query.get_full_url("html")
-            == "/data_browser/query/app.model/fa+1,fd-0,fn.html?bob__equals=fred&limit=123"
+        expected = (
+            "/data_browser/query/app.model/fa+1,fd-0,fn.html?bob__equals=fred&limit=123"
         )
+        assert query.get_full_url("html") == expected
 
     def test_url_no_filters(self, query):
         query.filters = []
@@ -325,29 +341,31 @@ class TestType:
         assert repr(StringType) == "StringType"
 
 
-class TestRegexType:
+class TestRegexType(ParseHelpers):
+    type_ = RegexType
+
     @pytest.mark.django_db
     @pytest.mark.parametrize(
-        "value,expected,err",
-        [
-            (".*", ".*", None),
-            ("\\", None, "Invalid regex"),
-        ],
+        "value,expected,err", [(".*", ".*", None), ("\\", None, "Invalid regex")]
     )
     def test_parse(self, value, expected, err):
-        parse_helper(RegexType, None, value, expected, err)
+        self.check_parse(value, expected, err)
 
 
-class TestStringType:
+class TestStringType(ParseHelpers):
+    type_ = StringType
+
     def test_default_lookup(self):
         assert StringType.default_lookup == "equals"
 
-    def test_format(self):
-        assert StringType.get_formatter(None)("bob") == "bob"
-        assert StringType.get_formatter(None)(None) is None
+    @pytest.mark.parametrize("value,expected", [("bob", "bob"), (None, None)])
+    def test_format(self, value, expected):
+        self.check_format(value, expected)
 
 
-class TestNumberType:
+class TestNumberType(ParseHelpers):
+    type_ = NumberType
+
     @pytest.mark.parametrize(
         "value,expected,err",
         [
@@ -356,29 +374,36 @@ class TestNumberType:
         ],
     )
     def test_parse(self, value, expected, err):
-        parse_helper(NumberType, None, value, expected, err)
+        self.check_parse(value, expected, err)
 
     def test_default_lookup(self):
-        assert NumberType.default_lookup == "equals"
+        assert self.type_.default_lookup == "equals"
 
-    def test_format(self):
-        assert NumberType.get_formatter(None)(6) == 6
-        assert NumberType.get_formatter(None)(None) is None
+    @pytest.mark.parametrize("value,expected", [(6, 6), (None, None)])
+    def test_format(self, value, expected):
+        self.check_format(value, expected)
 
 
 def dt(Y, M, D, h=0, m=0, s=0):
     return datetime(Y, M, D, h, m, s, tzinfo=timezone.utc)
 
 
-class TestDateTimeType:
+class TestDateTimeType(ParseHelpers):
+    type_ = DateTimeType
+
     @time_machine.travel("2020-12-13 09:42:53", tick=False)
     @pytest.mark.parametrize(
         "value,expected,err",
         [
             ("2018-03-20T22:31:23", dt(2018, 3, 20, 22, 31, 23), None),
             ("2018-3-2T22:31:23", dt(2018, 3, 2, 22, 31, 23), None),
+            ("2018-3-2 22:31:23", dt(2018, 3, 2, 22, 31, 23), None),
+            ("2018-3-2 days+1 22:31:23", dt(2018, 3, 3, 22, 31, 23), None),
+            ("2018-3-2 days+1 2:31", dt(2018, 3, 3, 2, 31, 0), None),
             ("hello", None, "Unrecognized clause 'hello'"),
             ("now", dt(2020, 12, 13, 9, 42, 53), None),
+            ("now hours+1", dt(2020, 12, 13, 10, 42, 53), None),
+            ("today", dt(2020, 12, 13, 0, 0, 0), None),
             # dateutil.parser
             ("11-11-2018", dt(2018, 11, 11), None),
             ("11-22-2018", dt(2018, 11, 22), None),
@@ -432,33 +457,39 @@ class TestDateTimeType:
         ],
     )
     def test_parse(self, value, expected, err):
-        parse_helper(DateTimeType, None, value, expected, err)
+        self.check_parse(value, expected, err)
 
     def test_default_lookup(self):
         assert DateTimeType.default_lookup == "equals"
 
     @pytest.mark.django_db
-    def test_aware_format(self, settings):
+    @pytest.mark.parametrize(
+        "value,expected",
+        [
+            (
+                datetime(2020, 5, 19, 8, 42, 16, tzinfo=timezone.utc),
+                "2020-05-19 08:42:16",
+            ),
+            (None, None),
+        ],
+    )
+    def test_aware_format(self, value, expected, settings):
         settings.USE_TZ = True
-        assert (
-            DateTimeType.get_formatter(None)(
-                datetime(2020, 5, 19, 8, 42, 16, tzinfo=timezone.utc)
-            )
-            == "2020-05-19 08:42:16"
-        )
-        assert DateTimeType.get_formatter(None)(None) is None
+        self.check_format(value, expected)
 
     @pytest.mark.django_db
-    def test_naive_format(self, settings):
+    @pytest.mark.parametrize(
+        "value,expected",
+        [(datetime(2020, 5, 19, 8, 42, 16), "2020-05-19 08:42:16"), (None, None)],
+    )
+    def test_naive_format(self, value, expected, settings):
         settings.USE_TZ = False
-        assert (
-            DateTimeType.get_formatter(None)(datetime(2020, 5, 19, 8, 42, 16))
-            == "2020-05-19 08:42:16"
-        )
-        assert DateTimeType.get_formatter(None)(None) is None
+        self.check_format(value, expected)
 
 
-class TestDurationType:
+class TestDurationType(ParseHelpers):
+    type_ = DurationType
+
     @pytest.mark.parametrize(
         "value,expected,err",
         [
@@ -468,47 +499,58 @@ class TestDurationType:
         ],
     )
     def test_parse(self, value, expected, err):
-        parse_helper(DurationType, None, value, expected, err)
+        self.check_parse(value, expected, err)
 
     def test_default_lookup(self):
-        assert DurationType.default_lookup == "equals"
+        assert self.type_.default_lookup == "equals"
 
-    def test_format(self):
-        assert (
-            DurationType.get_formatter(None)(timedelta(days=5, minutes=6))
-            == "5 days, 0:06:00"
-        )
-        assert DurationType.get_formatter(None)(timedelta(minutes=6)) == "0:06:00"
-        assert DurationType.get_formatter(None)(None) is None
+    @pytest.mark.parametrize(
+        "value,expected",
+        [
+            (timedelta(days=5, minutes=6), "5 days, 0:06:00"),
+            (timedelta(minutes=6), "0:06:00"),
+            (None, None),
+        ],
+    )
+    def test_format(self, value, expected):
+        self.check_format(value, expected)
 
 
-class TestDateType:
+class TestDateType(ParseHelpers):
+    type_ = DateType
+
     @time_machine.travel("2020-12-13 09:42:53", tick=False)
     @pytest.mark.parametrize(
         "value,expected,err",
         [
-            ("2018-03-20T22:31:23", dt(2018, 3, 20).date(), None),
+            ("2018-03-20T22:31:23", date(2018, 3, 20), None),
             ("hello", None, "Unrecognized clause 'hello'"),
-            ("today", dt(2020, 12, 13).date(), None),
-            ("11-22-2018", dt(2018, 11, 22).date(), None),
-            ("21-12-2018", dt(2018, 12, 21).date(), None),
+            ("today", date(2020, 12, 13), None),
+            ("today days+1", date(2020, 12, 14), None),
+            ("now", date(2020, 12, 13), None),
+            ("11-22-2018", date(2018, 11, 22), None),
+            ("21-12-2018", date(2018, 12, 21), None),
             ("11-12-2018", None, "Ambiguous value"),
             ("21-22-2018", None, "Unrecognized clause '21-22-2018'"),
-            ("days+1", dt(2020, 12, 14).date(), None),
+            ("days+1", date(2020, 12, 14), None),
         ],
     )
     def test_parse(self, value, expected, err):
-        parse_helper(DateType, None, value, expected, err)
+        self.check_parse(value, expected, err)
 
     def test_default_lookup(self):
-        assert DateType.default_lookup == "equals"
+        assert self.type_.default_lookup == "equals"
 
-    def test_format(self):
-        assert DateType.get_formatter(None)(date(2020, 5, 19)) == "2020-05-19"
-        assert DateType.get_formatter(None)(None) is None
+    @pytest.mark.parametrize(
+        "value,expected", [(date(2020, 5, 19), "2020-05-19"), (None, None)]
+    )
+    def test_format(self, value, expected):
+        self.check_format(value, expected)
 
 
-class TestBooleanType:
+class TestBooleanType(ParseHelpers):
+    type_ = BooleanType
+
     @pytest.mark.parametrize(
         "value,expected,err",
         [
@@ -518,86 +560,84 @@ class TestBooleanType:
         ],
     )
     def test_parse(self, value, expected, err):
-        parse_helper(BooleanType, None, value, expected, err)
+        self.check_parse(value, expected, err)
 
-    def test_format(self):
-        assert BooleanType.get_formatter(None)(None) is None
+    @pytest.mark.parametrize("value,expected", [(None, None)])
+    def test_format(self, value, expected):
+        self.check_format(value, expected)
 
     def test_default_lookup(self):
-        assert BooleanType.default_lookup == "equals"
+        assert self.type_.default_lookup == "equals"
 
 
-class TestStringChoiceType:
+class TestStringChoiceType(ParseHelpers):
+    type_ = StringChoiceType
     choices = [("a", "A"), ("b", "B"), ("c", "C")]
 
-    def test_format(self):
-        assert StringChoiceType.get_formatter(self.choices)("b") == "B"
-        assert StringChoiceType.get_formatter(self.choices)(None) is None
+    @pytest.mark.parametrize("value,expected", [("b", "B"), (None, None)])
+    def test_format(self, value, expected):
+        self.check_format(value, expected)
 
     def test_bad_value(self):
-        assert StringChoiceType.get_formatter(self.choices)("x") == "x"
+        assert self.type_.get_formatter(self.choices)("x") == "x"
 
     @pytest.mark.parametrize(
-        "value,expected,err",
-        [
-            ("B", "b", None),
-            ("X", None, "Unknown choice 'X'"),
-        ],
+        "value,expected,err", [("B", "b", None), ("X", None, "Unknown choice 'X'")]
     )
     def test_parse(self, value, expected, err):
-        parse_helper(NumberChoiceType, self.choices, value, expected, err)
+        self.check_parse(value, expected, err)
 
 
-class TestNumberChoiceType:
+class TestNumberChoiceType(ParseHelpers):
+    type_ = NumberChoiceType
     choices = [(1, "A"), (2, "B"), (3, "C")]
 
-    def test_format(self):
-        assert NumberChoiceType.get_formatter(self.choices)(2) == "B"
-        assert NumberChoiceType.get_formatter(self.choices)(None) is None
+    @pytest.mark.parametrize("value,expected", [(2, "B"), (None, None)])
+    def test_format(self, value, expected):
+        self.check_format(value, expected)
 
     def test_bad_value(self):
-        assert NumberChoiceType.get_formatter(self.choices)(6) == 6
+        assert self.type_.get_formatter(self.choices)(6) == 6
 
     @pytest.mark.parametrize(
-        "value,expected,err",
-        [
-            ("B", 2, None),
-            ("X", None, "Unknown choice 'X'"),
-        ],
+        "value,expected,err", [("B", 2, None), ("X", None, "Unknown choice 'X'")]
     )
     def test_parse(self, value, expected, err):
-        parse_helper(NumberChoiceType, self.choices, value, expected, err)
+        self.check_parse(value, expected, err)
 
 
-class TestHTMLType:
-    def test_format(self):
-        assert HTMLType.get_formatter(None)(None) is None
+class TestHTMLType(ParseHelpers):
+    type_ = HTMLType
+
+    @pytest.mark.parametrize("value,expected", [(None, None)])
+    def test_format(self, value, expected):
+        self.check_format(value, expected)
 
 
-class TestIsNullType:
-    def test_format(self):
-        assert IsNullType.get_formatter(IsNullType.choices)(True) == "IsNull"
-        assert IsNullType.get_formatter(IsNullType.choices)(False) == "NotNull"
-        assert IsNullType.get_formatter(IsNullType.choices)(None) == "IsNull"
+class TestIsNullType(ParseHelpers):
+    type_ = IsNullType
 
     @pytest.mark.parametrize(
-        "value,expected,err",
-        [
-            ("IsNull", True, None),
-            ("NotNull", False, None),
-        ],
+        "value,expected", [(True, "IsNull"), (False, "NotNull"), (None, "IsNull")]
+    )
+    def test_format(self, value, expected):
+        self.check_format(value, expected)
+
+    @pytest.mark.parametrize(
+        "value,expected,err", [("IsNull", True, None), ("NotNull", False, None)]
     )
     def test_parse(self, value, expected, err):
-        parse_helper(IsNullType, IsNullType.choices, value, expected, err)
+        self.check_parse(value, expected, err)
 
 
-class TestUUIDType:
+class TestUUIDType(ParseHelpers):
+    type_ = UUIDType
     s = "210081a4-7315-4173-aa73-4bab47b2f9a5"
     u = UUID(s)
 
-    def test_format(self):
-        assert UUIDType.get_formatter(None)(self.u) == self.s
-        assert UUIDType.get_formatter(None)(None) is None
+    @pytest.mark.parametrize("value,expected", [(u, s), (None, None)])
+    def test_format(self, value, expected):
+        self.check_format(value, expected)
 
     @pytest.mark.parametrize(
         "value,expected,err",
@@ -608,61 +648,71 @@ class TestUUIDType:
         ],
     )
     def test_parse(self, value, expected, err):
-        parse_helper(UUIDType, None, value, expected, err)
+        self.check_parse(value, expected, err)
 
 
-class TestUnknownType:
-    def test_format(self):
-        assert UnknownType.get_formatter(None)(None) is None
+class TestUnknownType(ParseHelpers):
+    type_ = UnknownType
+
+    @pytest.mark.parametrize("value,expected", [(None, None)])
+    def test_format(self, value, expected):
+        self.check_format(value, expected)
 
 
-class TestStringArrayType:
+class TestStringArrayType(ParseHelpers):
+    type_ = StringArrayType
+
     @pytest.mark.parametrize(
         "value,expected,err",
-        [
-            ('["bob"]', ["bob"], None),
-            ('"bob"', None, "Expected a list"),
-        ],
+        [('["bob"]', ["bob"], None), ('"bob"', None, "Expected a list")],
     )
     def test_parse(self, value, expected, err):
-        parse_helper(StringArrayType, None, value, expected, err)
+        self.check_parse(value, expected, err)
 
 
-class TestStringChoiceArrayType:
+class TestStringChoiceArrayType(ParseHelpers):
+    type_ = StringChoiceArrayType
     choices = [("fred", "bob")]
 
     @pytest.mark.parametrize(
         "value,expected,err",
-        [
-            ('["bob"]', ["fred"], None),
-            ('"bob"', None, "Expected a list"),
-        ],
+        [('["bob"]', ["fred"], None), ('"bob"', None, "Expected a list")],
     )
     def test_parse(self, value, expected, err):
-        parse_helper(StringChoiceArrayType, self.choices, value, expected, err)
+        self.check_parse(value, expected, err)
 
-
-class TestNumberArrayType:
     @pytest.mark.parametrize(
-        "value,expected,err",
-        [
-            ("[123]", [123], None),
-            ("123", None, "Expected a list"),
-        ],
+        "lookup,value,expected,err",
+        [("contains", "bob", "fred", None), ("length", "1", 1, None)],
+    )
+    def test_parse_lookup(self, lookup, value, expected, err):
+        self.check_parse_lookup(lookup, value, expected, err)
+
+
+class TestNumberArrayType(ParseHelpers):
+    type_ = NumberArrayType
+
+    @pytest.mark.parametrize(
+        "value,expected,err", [("[123]", [123], None), ("123", None, "Expected a list")]
     )
     def test_parse(self, value, expected, err):
-        parse_helper(NumberArrayType, None, value, expected, err)
+        self.check_parse(value, expected, err)
 
 
-class TestNumberChoiceArrayType:
+class TestNumberChoiceArrayType(ParseHelpers):
+    type_ = NumberChoiceArrayType
     choices = [(123, "bob")]
 
     @pytest.mark.parametrize(
         "value,expected,err",
-        [
-            ('["bob"]', [123], None),
-            ('"bob"', None, "Expected a list"),
-        ],
+        [('["bob"]', [123], None), ('"bob"', None, "Expected a list")],
     )
     def test_parse(self, value, expected, err):
-        parse_helper(NumberChoiceArrayType, self.choices, value, expected, err)
+        self.check_parse(value, expected, err)
+
+    @pytest.mark.parametrize(
+        "lookup,value,expected,err",
+        [("contains", "bob", 123, None), ("length", "1", 1, None)],
+    )
+    def test_parse_lookup(self, lookup, value, expected, err):
+        self.check_parse_lookup(lookup, value, expected, err)
